/**************************************************************************************************
  QuadDecoder.c
  Quadrature decoder state machine.
**************************************************************************************************/

// Include Files
#include "QuadDecoder.h"
#include "Buffers.h"
#include "USB_Driver.h"
#include "Common.h"

// Variables
char QD_State = 0x00;                  // State of Quad Decoder
char A = 0;                            // Shaft encoder shadow value of input A
char B = 0;                            // Shaft encoder shadow value of input B
unsigned int CountValue = 0;           // Counter incremented/decremented by quadrature decoder



/**************************************************************************************************
  External interrupt handler for ports PA0 and PA1 (Shaft encoder A and shaft encoder B).
**************************************************************************************************/
void ExternalInterrupt0() iv IVT_INT_EXTI0 ics ICS_AUTO {

     PR0_bit = 1;                      // Clear interrupt
     
     if (IDR0_GPIOA_IDR_bit == 1) {
         A = 1;
         USB_DEBUGOUT("Int0 A=1\r\n");
     } else {
         A = 0;
         USB_DEBUGOUT("Int0 A=0\r\n");
     }
}

void ExternalInterrupt1() iv IVT_INT_EXTI1 ics ICS_AUTO {

     PR1_bit = 1;                      // Clear interrupt

     if (IDR1_GPIOA_IDR_bit == 1) {
         B = 1;
         USB_DEBUGOUT("Int1 B=1\r\n");
     } else {
         B = 0;
         USB_DEBUGOUT("Int1 B=0\r\n");
     }
}

/**************************************************************************************************
  Initialize shaft encoder inputs pins.
**************************************************************************************************/
void Ext_Int_Init(void) {
     char *BufNum;
     
     // Configure PA0 as input with pull-up
     GPIO_Set_Pin_Mode(&GPIOA_BASE, _GPIO_PIN_0, _GPIO_CFG_MODE_INPUT | _GPIO_CFG_PULL_UP);

     // Configure PA1 as input with pull-up
     GPIO_Set_Pin_Mode(&GPIOA_BASE, _GPIO_PIN_1, _GPIO_CFG_MODE_INPUT | _GPIO_CFG_PULL_UP);
     
     // Read shaft encoder A and B channels and set shadow values
     A = IDR0_GPIOA_IDR_bit;
     B = IDR1_GPIOA_IDR_bit;
     
     // Diagnostic
     BufNum = GetBuffer();
     sprintf(BufNum,"A: %d  B: %d\r\n",A,B);
     USB_DEBUGOUT(BufNum);

     // Set the decoder phase appropriately
     if ((A == 0) && (B == 0))
         QD_State = 0x00;
     else if ((A == 1) && (B == 0))
         QD_State = 0x05;
     else if ((A == 1) && (B == 1))
         QD_State = 0x0F;
     else if ((A == 0) && (B == 1))
         QD_State = 0x0A;

     // Set EXTI_IMR register
     MR0_bit = 1;                      // Unmask interrupt request from line 0

     // Set EXTI_IMR register
     MR1_bit = 1;                      // Unmask interrupt request from line 1
     
     
     // Set EXTI_RTSR register
     TR0_bit = 1;                      // Enable rising trigger

     // Set EXTI_FTSR register
     TR0_EXTI_FTSR_bit = 1;            // Enable falling trigger

     // Set EXTI_RTSR register
     TR1_bit = 1;                      // Enable rising trigger

     // Set EXTI_FTSR register
     TR1_EXTI_FTSR_bit = 1;            // Enable falling trigger
     
     
     
     // Set SYSCFGEN bit in RCC_APB2ENR register
     SYSCFGEN_bit = 1;                 // Enable system configuration controller clock

     // Set SYSCFG_EXTICR1
     SYSCFG_EXTICR1 = 0x00000000;      // Connect PA0 to EXTI0 PA1 to EXTI1

     // Enable external interrupts
     NVIC_IntEnable(IVT_INT_EXTI0);    // Enable EXTI0 interrupt
     NVIC_IntEnable(IVT_INT_EXTI1);    // Enable EXTI1 interrupt

}

/**************************************************************************************************
  Shaft encoder processing. Add your own code here.
**************************************************************************************************/
void CountUp(void) {
     char *BufNum;
     
     if (CountValue < 9999)
         CountValue += 1;
     else
         CountValue = 0;

     // Diagnostic
     BufNum = GetBuffer();
     sprintf(BufNum,"CountValue: %d\r\n",CountValue);
     USB_PutQueue(BufNum);
}

void CountDown(void) {
     char *BufNum;
     
     if (CountValue > 0)
         CountValue -= 1;
     else
         CountValue = 9999;

     // Diagnostic
     BufNum = GetBuffer();
     sprintf(BufNum,"CountValue: %d\r\n",CountValue);
     USB_PutQueue(BufNum);
}




/**************************************************************************************************
Shaft encoder input:

     ___________             ___________             ___________             ___________
 A  |           |           |           |           |           |           |           |
____|           |___________|           |___________|           |___________|           |__________
           ___________      |      ___________      |      ___________      |      ___________
 B        |           |     |     |     |     |     |     |     |     |     |     |           |
__________|           |___________|     |     |___________|     |     |___________|           |____
                      |     |     |     |     |     |     |     |     |     |
                      |!A&!B|A&!B | A&B |!A&B |!A&!B|A&!B | A&B |!A&B |!A&!B|

                                    Down <---------------> Up


State machine sequence for forward and reverse shaft encoder direction:


                   Bits [1:0] of QD_State
                     00   01   11   10
                   +----+----+----+----+
               00  | 0  | 1  | 3  | 2  |        Up count sequence:
                   +----+----+----+----+        0 -> 1 -> 5 -> 7 -> F -> E -> A -> 2 -> 0
               01  | 4  | 5  | 7  | 6  |
 Bits [3:2]        +----+----+----+----+
of QD_State    11  | C  | D  | F  | E  |        Down count sequence:
                   +----+----+----+----+        0 -> 8 -> A -> B -> F -> D -> 5 -> 4 -> 0
               10  | 8  | 9  | B  | A  |
                   +----+----+----+----+


State diagram:
               +--------------------------------------------------------------+
               |                                                              |
               |      A&!B +-----+    A&B  +-----+    !A&B +-----+    !A&!B+-----+
               |      +--->|  1  |    +--->|  7  |    +--->|  E  |    +--->|  2  |
               |      |    +-----+    |    +-----+    |    +-----+    |    +-----+
               |      |         |     |         |     |         |     |
               |      |         |     |         |     |         |     |
               |   +-----+      |  +-----+      |  +-----+      |  +-----+
               |   |  0  |      |  |  5  |      |  |  F  |      |  |  A  |
               +-->|while|<-+   +->|while|<-+   +->|while|<-+   +->|while|<--+
                   |!A&!B|  |      |A&!B |  |      | A&B |  |      |!A&B |   |
                   +-----+  |      +-----+  |      +-----+  |      +-----+   |
                      |     |         |     |         |     |         |      |
                      |     |         |     |         |     |         |      |
           +-----+    |    +-----+    |    +-----+    |    +-----+    |      |
           |  8  |<---+    |  4  |<---+    |  D  |<---+    |  B  |<---+      |
           +-----+ !A&B    +-----+!A&!B    +-----+ A&!B    +-----+  A&B      |
              |                                                              |
              +--------------------------------------------------------------+




**************************************************************************************************/
void Decoder_SM(void) {

     switch (QD_State) {
     
         case 0x00: // !A && !B
             if (A && !B) { // Up direction
                 USB_DEBUGOUT("Going to state 0x01");
                 QD_State = 0x01;
             }
             if (!A && B) { // Down direction
                 USB_DEBUGOUT("Going to state 0x08");
                 QD_State = 0x08;
             }
             break;
     
         case 0x01: // Up direction execution
             // Do something here
             CountUp();
             
             USB_DEBUGOUT("Going to state 0x05");
             QD_State = 0x05;
             break;
     
         case 0x02: // Up direction execution
             // Do something here
             CountUp();
             
             USB_DEBUGOUT("Going to state 0x00");
             QD_State = 0x00;
             break;
     
         case 0x03: // Invalid state
             USB_PutQueue("Should not be in unused state 0x03");
             QD_State = 0x00;
             break;
             
         case 0x04: // Down direction execution
             // Do something here
             CountDown();
             
             USB_DEBUGOUT("Going to state 0x00");
             QD_State = 0x00;
             break;
             
         case 0x05: // A && !B
             if (A && B) { // Up direction
                 USB_DEBUGOUT("Going to state 0x07");
                 QD_State = 0x07;
             }
             if (!A && !B) { // Down direction
                 USB_DEBUGOUT("Going to state 0x04");
                 QD_State = 0x04;
             }
             break;
             
         case 0x06: // Invalid state
             USB_PutQueue("Should not be in unused state 0x06");
             QD_State = 0x00;
             break;
             
         case 0x07: // Up direction execution
             // Do something here
             CountUp();
             
             USB_DEBUGOUT("Going to state 0x0F");
             QD_State = 0x0F;
             break;
             
         case 0x08: // Down direction execution
             // Do something here
             CountDown();
             
             USB_DEBUGOUT("Going to state 0x0A");
             QD_State = 0x0A;
             break;
             
         case 0x09: // Invalid state
             USB_PutQueue("Should not be in unused state 0x09");
             QD_State = 0x00;
             break;
             
         case 0x0A: // !A && B
             if (!A && !B) { // Up direction
                 USB_DEBUGOUT("Going to state 0x02");
                 QD_State = 0x02;
             }
             if (A && B) { // Down direction
                 USB_DEBUGOUT("Going to state 0x0B");
                 QD_State = 0x0B;
             }
             break;
             
         case 0x0B: // Down direction execution
             // Do something here
             CountDown();
             
             USB_DEBUGOUT("Going to state 0x0F");
             QD_State = 0x0F;
             break;
             
         case 0x0C: // Invalid state
             USB_PutQueue("Should not be in unused state 0x0C");
             QD_State = 0x00;
             break;
         
         case 0x0D: // Down direction execution
             // Do something here
             CountDown();
             
             USB_DEBUGOUT("Going to state 0x05");
             QD_State = 0x05;
             break;
             
         case 0x0E: // Up direction execution
             // Do something here
             CountUp();
             
             USB_DEBUGOUT("Going to state 0x0A");
             QD_State = 0x0A;
             break;
         
         case 0x0F: // A && B
             if (!A && B) { // Up direction
                 USB_DEBUGOUT("Going to state 0x0E");
                 QD_State = 0x0E;
             }
             if (A && !B) { // Down direction
              USB_DEBUGOUT("Going to state 0x0D");
              QD_State = 0x0D;
             }
             break;
         
         default: // Any other invalid state
             USB_PutQueue("Should not be in any other state");
             QD_State = 0x00;
             break;
             
     }
}




























/**************************************************************************************************
  EOF
**************************************************************************************************/